const path = require("path");

const safeRequire = (file) => {
    console.log(file)
    try {
        const required = require(file);
        return required;
    } catch(err) {
        return false
    }
}

// the lower down the object the query is, the higher priority it has.
// e.g. webpack prod and dev files should be below the base as they have a higher priority
const webpackQueries = {
    WEBPACK_FILE (config, filename, prod, projectPath) {
        const updates = safeRequire(path.resolve(projectPath, filename));
        return updates ? updates : {};
    },
    WEBPACK_PROD_FILE (config, filename, prod, projectPath) {
        const updates = safeRequire(path.resolve(projectPath, filename));
        return prod && updates ? updates : {};
    },
    WEBPACK_DEV_FILE (config, filename, prod, projectPath) {
        const updates = safeRequire(path.resolve(projectPath, filename));
        return !prod && updates ? updates : {};
    }
}

const settingsQueries = {
    SETTINGS_OPEN_BROWSER (config, value) {
        return {dontOpenBrowser: value === "false"};
    }
}

// clojure, returns a function that will be used  
const createConfigTweaker = queries => {
    const getQueries = () => queries;
    return (initialConfig={}, prod=false, projectPath=path.resolve("./")) => {
        const config = Object.keys(getQueries())
            .reduce((config, key) => {
                const value = process.env[key];
                if (value) { // if key used in the .env file
                    const modifications = getQueries()[key](config, value, prod, projectPath) || {}; // call the query function or return an empty object
                    return Object.assign({}, config, modifications);
                }
                // if the user didn't use the particular key in the env file, then don't make any changes
                return config;

            }, initialConfig);
        return config;
    }
}

// Generates a new webpack config tweaker
const webpackConfigTweaker = createConfigTweaker(webpackQueries);


const settingsConfigTweaker = createConfigTweaker(settingsQueries);


module.exports = {
    webpack: webpackConfigTweaker,
    settings: settingsConfigTweaker,
};